#### Sample code for size analysis when N = 10, true DGP: iid, unequal pairwise correlations
# Note:
# 1. one need to change variable "l" to do size analysis for N = 3,5
# 2. change variable "var" for different variances in the second half of the sample
# 3. use grid seq(0.1,2,by=0.1) for "var" will affect the random seed, need put set.seed() into loop, then not able to reproduce the result in paper
# 4. clustersetRNGStream(1) during parallel computation
# 5. computation across dimensions are always done by loops, i.e., inside the function "cusumtest" or bootstrapping 
rm(list=ls())
library(mvtnorm)
#### PATH
if (.Platform$OS.type == "unix") {
  # Mac
  path <- c("/Users/fangduan/Dropbox/Dissertation/Correlation Test/R/")
} else if (.Platform$OS.type == "windows"){
  # Windows
  path <- c("C:/Users/Win7Adm/Dropbox/Dissertation/Correlation Test/R/")
}
setwd(path)
argmaxest <- function(X)
{
  n <- length(X)
  vektor <- rep(0,n)
  Vari <- rep(0,n)
  varn <- var(X)
  Vari <- cumsum(X^2)/(1:n) - (cumsum(X)/(1:n))^2
  ##Vari[i] <- mean((X[1:i])^2)
  vektor <- (1:n)/sqrt(n) * (Vari - varn)
  result <- which.max(abs(vektor))
  return(result)
}

## Function for the new procedure
cusumtest <- function(data, vec.est=rep(0,10), est=T)
{
  n <- nrow(data)
  l <- ncol(data)
  
  if(est==T){vec.est <- apply(data,2,argmaxest)}
  for (i in 1:length(vec.est))
  {
    if (vec.est[i] >= n-1){
      vec.est[i] <- n-2
    } else if (vec.est[i] == 1){
      vec.est[i] <- 2
    }
  }
  
  clean <- matrix(NA,ncol=l,nrow=n)
  for (ii in 1:l){
    if (sd(data[1:vec.est[ii],ii]) * sd(data[-(1:vec.est[ii]),ii]) >0 ){
      clean[1:vec.est[ii],ii] <- data[1:vec.est[ii],ii] / sd(data[1:vec.est[ii],ii])
      clean[-(1:vec.est[ii]),ii] <- data[-(1:vec.est[ii]),ii] / sd(data[-(1:vec.est[ii]),ii])
    } else if (sd(data[1:vec.est[ii],ii]) * sd(data[-(1:vec.est[ii]),ii]) == 0){
      clean[,ii] <- data[,ii] / sd(data[,ii])
    }
  }
  
  for (j in 1:(l-1)){
    for (k in (j+1):l){
      lhs <- paste("x",j,k,sep="")
      rhs <- paste("clean[,",j,"]*clean[,",k,"]",sep="")
      eq <- paste(lhs,rhs,sep=" <- ")
      eval(parse(text=eq))
    }
  }
  
  if (l == 10){
    x <- cbind(x12,x13,x14,x15,x16,x17,x18,x19,x110,
               x23,x24,x25,x26,x27,x28,x29,x210,
               x34,x35,x36,x37,x38,x39,x310,
               x45,x46,x47,x48,x49,x410,
               x56,x57,x58,x59,x510,
               x67,x68,x69,x610,
               x78,x79,x710,
               x89,x810,
               x910)
  } else if (l == 5){
    x <- cbind(x12,x13,x14,x15,x23,x24,x25,x34,x35,x45)
  } else if (l == 3){
    x <- cbind(x12,x13,x23)
  }
  
  Omega <- cov(x)
  
  S <- apply(x,2,function(x) cumsum(x)-(1:n)/n * sum(x))
  process <- sapply(1:nrow(S),function(s) 1/sqrt(n) * sqrt( S[s,] %*% solve(Omega) %*% as.matrix(S[s,])) )
  
  result <- abs(process)
  
  return(max(result))
}

set.seed(1)
m <- 1000
n <- 500 # n <- 1000
B <- 199
l <- 10 # l <- 3,5
var <- 0.9 # grid of variance: seq(0.1,2,by=0.1)
rejprob <- matrix(NA,ncol=2,nrow=length(var))

bootpvals_unknown = rep(0,m)
bootpvals_known = rep(0,m)

for (i in 1:m)
{
  print(i)
  if (l == 10){
  corr.mat <- matrix(c(1,0.4,0.4,0.5,0.5,0.5,0.6,0.6,0.7,0.7,
                       0.4,1,0.5,0.5,0.5,0.6,0.6,0.6,0.5,0.5,
                       0.4,0.5,1,0.4,0.4,0.4,0.5,0.5,0.6,0.6,
                       0.5,0.5,0.4,1,0.5,0.5,0.4,0.5,0.4,0.4,
                       0.5,0.5,0.4,0.5,1,0.6,0.5,0.6,0.5,0.4,
                       0.5,0.6,0.4,0.5,0.6,1,0.4,0.5,0.6,0.5,
                       0.6,0.6,0.5,0.4,0.5,0.4,1,0.4,0.5,0.4,
                       0.6,0.6,0.5,0.5,0.6,0.5,0.4,1,0.5,0.6,
                       0.7,0.5,0.6,0.4,0.5,0.6,0.5,0.5,1,0.5,
                       0.7,0.5,0.6,0.4,0.4,0.5,0.4,0.6,0.5,1),ncol=l)
  } else if (l == 5){
    corr.mat <- matrix(c(1,0.4,0.7,0.1,0.2,0.4,1,0.5,0.6,0.2,0.7,0.5,1,0.1,0.3,0.1,0.6,0.1,1,0.1,0.2,0.2,0.3,0.1,1),ncol=l)
  } else if (l == 3){
    corr.mat <- matrix(c(1,0.4,0.7,0.4,1,0.5,0.7,0.5,1),ncol=l)
  }
  
  data1 <- rmvnorm(n/2,rep(0,l),corr.mat)
  data2 <- rmvnorm(n/2,rep(0,l),var*corr.mat)
  data <- rbind(data1,data2)
  sd1 <- rep(NA,l)
  sd2 <- rep(NA,l)
  vec_est <- apply(data,2,argmaxest)
  for (a in 1:length(vec_est))
  {
    if (vec_est[a] >= n-1){
      vec_est[a] <- n-2
    } else if (vec_est[a] == 1){
      vec_est[a] <- 2
    }
  }
  for (jj in 1:l){
    sd1[jj] <- sd(data[1:vec_est[jj],jj])
  }
  for (jj in 1:l){
    sd2[jj] <- sd(data[-(1:vec_est[jj]),jj])
  }
  
  test_unknown <- cusumtest(data)
  test_known <- cusumtest(data, vec.est = rep(n/2,l),est=F)
  
  boot_distr_unknown = rep(0,B)
  boot_distr_known = rep(0,B)
  
  dataDemean <- apply(data,2,function(x) x-mean(x))
  for(b in 1:B)
  {
    index <- sample(1:n,n,replace=T)
    xboot <- dataDemean[index,]
    
    for (kk in 1:l)
    {
      if(sd(xboot[1:vec_est[kk],kk]) * sd(xboot[-(1:vec_est[kk]),kk]) > 0)
      {xboot[1:vec_est[kk],kk] <- xboot[1:vec_est[kk],kk] / sd(xboot[1:vec_est[kk],kk]) * sd1[kk]
      xboot[-(1:vec_est[kk]),kk] <- xboot[-(1:vec_est[kk]),kk] / sd(xboot[-(1:vec_est[kk]),kk]) * sd2[kk]
      }
      if(sd(xboot[1:vec_est[kk],kk]) * sd(xboot[-(1:vec_est[kk]),kk]) == 0)
      {xboot[,kk] <- xboot[,kk] / sd(xboot[,kk])}
    }
    
    boot_distr_unknown[b] = cusumtest(xboot)
    boot_distr_known[b] = cusumtest(xboot,vec.est = rep(n/2,l),est=F)
  }
  pval_unknown = mean(test_unknown <= boot_distr_unknown,na.rm=T)
  pval_known = mean(test_known <= boot_distr_known,na.rm=T)
  
  bootpvals_unknown[i] = pval_unknown 
  bootpvals_known[i] = pval_known 
}
rejprob[1,1] <- mean(bootpvals_unknown < 0.05,na.rm=T)
rejprob[1,2] <- mean(bootpvals_known < 0.05,na.rm=T)
write.table(rejprob,paste("iid_rejprob_size_dim_",l,"_var_",var,"_T_",n,"_unequal.txt",sep=""))
